#!/usr/bin/perl -w

###################################################################
##
## Wichtige Informationen
##
## Zu jedem Projekt wird versucht, die projektDatei: <projektName>.cfg
## zu ffnen und auszulesen. Dies geschieht in webFrame::fileOperation .
##
## Rechte:
## Jeder User verfgt ber Rechte, die in der USER DB abgelegt werden.
## Das Recht 0 deutet auf einen GastUser, der nur ansehen darf.
## Das Recht 1 deutet auf einen Admin und fhrt zur Admin Seite.
## Alle anderen Rechte knnen beliebig ausgewertet werden.
##
## Das sub Main kommuniziert mit allen aufgerufenen Seiten ber ein Hash.
## In diesem stehen jeweils die bentigten Daten drinn (oft auch andere Hashes).
##
## Um das Script an eigene WebAnwendungen anzupassen sind zwei nderungen
## notwendig. Einmal in dem Script webFrameMain.pl und einmal in buildHTMLPage.pm
##
## Die UserID mu fr die user.db eindeutig sein.
##
## Von jeder HTML Seiten werden folgende Parameter bergeben:
## sourcePage     => Name der Seite, die gerade den HTTP Request losschickt
## action         => Bezeichnung der erwarteten Aktion
## login          => LoginName, damit id oder passwd in der DB gegen geprft werden knnen
## passwd         => wenn es keine id gibt, gilt dieses
## sessionID      => bezeichnet die Session
## projekt        => bezeichnet das Projekt und damit das genutzte Verzeichniss, sowie das CFG File
##
## Die Lnge der sessionID ist zwar frei whlbar, wird aber aus praktischen Grnden auf den Bereich 
## 5 bis 50 beschrnkt. Da die Zeichen in der sessionID nicht gleichverteilt sind, sondern mit eine W
## von jeweils 1/3 klein/gro Buchstaben oder Ziffern gezogen werden, ist der erfolgsversprechendste
## Angriff, der, der nur Ziffern ausprobiert (da es von diesen nur 10 gibt, von den Buchstaben jeweils
## 26). Fr jede Stelle gibt es also eine W von 1/3 berhaupt den richtigen Bereich zu haben multipliziert
## mit einer W von 1/10 genau das richtige Zeichen zu erwischen, d.h. 1/30 pro Zeichen. Bei 10 Zeichen
## wren das also (1/30)^10 => lt Excel 1,694E-15 oder anders, nach ~ 600.000.000.000.000 Angriffen 
## htte man einen Treffer (erhht man die # der Stellen um eine, so erhhen sich die # der notwendigen
## Angriff um 30). ACHTUNG !! auch wenn diese Zahl hoch erscheinen mag, das Netz ermglicht massiv verteilte
## Angriffe, mit vielleicht bis zu 1.000.000 pro Sekunde. (In diesem Fall wrde der
## Angreifer noch 60.000.000 Sekunden, oder knapp 2 Jahre bentigen.)
## 
## Stolperfallen:
## Parameter der zentralen ConfigDatei drfen nicht genauso heien, wie die
## Parameter, die ber HTML zurck kommen (gilt natrlich auch vice versa).
##
##
###################################################################

## @todo:

## AdminPage speichern fehlt noch
## BeispielsAnwendung (DVD Datenbank)
## EMail Client


###################################################################
##
## Systemweite, wichtige Einstellungen, die vorhanden sein mssen,
## bevor das ConfigFile ausgelesen wird.
##
###################################################################

###############
## In readDataFromHTTP werden aus Sicherheitsgrnden alle Sonderzeichen
## aus den erhaltenen Daten (HTTP) entfernt, wenn hier eine 1 steht. Steht
## hier eine 0 werden die Sonderzeichen nicht entfernt. Bei einer 2 werden
## ein paar auserwhlte Sonderzeichen entfernt. ACHTUNG !! Es werden aus
## allen Feldern die Sonderzeichen entfernt, auch Formfelder mit z.B. Pfadangaben.
my $RESTRICT_DATA_FROM_HTML = "2";

###################################################################
##
## Module einbinden
##
###################################################################

use strict;

use webFrame::readDataFromHTTP;     # Auslesen der Daten, die ber HTTP ankommen
use webFrame::buildHTMLPage;        # Das erstellen der HTML Seiten
use webFrame::userHandling;         # 
use webFrame::fileOperation;        # Alle DateiOperationen
use webFrame::utility;              # Methoden die berall genutzt werden knnen.

###################################################################
##
## Ablauf: Einlesen, Verarbeiten, Ausgabe
##
###################################################################

###############
##### Einlesen
###############

## Daten aus dem ENV einlesen.
my %datenFromENV = %ENV;

## Daten von der Oberflche in ein Hash Einlesen
my $datenFromHTTPString = readDataFromHTTP(\%datenFromENV, $RESTRICT_DATA_FROM_HTML); # webFrame::readDataFromHTTP;

###############
##### Verarbeiten
###############

## ConfigFile einlesen
my $datenFromConfig = readDataFromFileHTTP($datenFromHTTPString); # use webFrame::fileOperation;

## Daten verarbeiten
my $datenFromMain = main($datenFromHTTPString, \%datenFromENV, $datenFromConfig);

###############
##### Ausgabe
###############

## HTML Seite bauen
my $htmlPage = buildHTMLPage($datenFromMain); # webFrame::buildHTMLPAGE;

## HTML Seite ausgeben
printHTMLPage($htmlPage);

###################################################################
##
## Programm Ende
##
###################################################################

exit;

###################################################################
##
## Unterprogramme
##
###################################################################

###################################################################
###################################################################
## Ausgabe der HTML Seite an den Aufrufer
sub printHTMLPage
{
      ## bernahme der Parameter
      my ($htmlPage) = @_;
      
      ## Notwendigen Kopf fr das HTTP
      print "Content-type: text/html\r\n\r\n";
      
      ## HTML Seite ausgeben
      print $htmlPage;

      return();
}

###################################################################
###################################################################
## Es gibt aber noch die Mglichkeit, das es zu einem unerwarteten
## Fehler kommt, dazu wird dann folgende Seite ausgegeben.
## Gerne genutzt bei allen Anweisungen, zu denen ein "or die .." gehrt.

sub printERRORPage
{
      ## bernahme der Parameter
      my ($ErrorMessage) = @_;
      
      ## Notwendiger Kopf fr das HTTP
      print "Content-type: text/html\r\n\r\n";
      
      ## HTML Anfang
      print "<html><head><title>WebFrame - all Data</title>\n";
      print "</head>\n";
      
      print "<BODY BGCOLOR=ffffff>";
      print "<h1>ERROR</h1>\n";
 
      print "<p>$ErrorMessage</p>";
 
      print "</body></html>\n";
      
      ## Auch wenn nicht klar ist, wohin.
      exit;     
}


###################################################################
###################################################################
## Main Programm zur Ablaufsteuerung

sub main
{
      ## bernahme der Parameter
      my ($datenFromHTTPString, $datenFromENVString, $datenFromConfigString) = @_;
      
      ## Auspacken
      my %datenFromHTTP = %{$datenFromHTTPString};
      my %datenFromENV = %{$datenFromENVString};
      my %datenFromConfig = %{$datenFromConfigString};
      my %datenFromUserDB;
      my %datenFromSUBMain;
      my %datenForTheHTMLEnv;
      
      ## Antwort
      my %hashDaten2Print;
      
      ## Variable, um festzustellen, ob sich der User korrekt angemeldet hat
      my $korrektAngemeldet = "no";
      
      ## Variable fr die sessionID
      my $sessionID;
      
      ############
      ## ANMELDUNG
      ############
      
      ## Um Massenangriffe zu erschweren, erfolgt hier eine
      ## Verzgerung von x Sekunden (einstellbar in der ConfigDatei).
      sleep($datenFromConfig{"wait"});
      
      ## Anmeldung erfolgt ber die sessionID
      if($datenFromHTTP{"sessionID"} ne "")
      {
            ## gibt es den USER in der DB, hier steht entweder ein
            ## "no" dann gibt es ihn nicht, oder ein Hash als String
            ## mit allen Werten
            my $userDataString = getUserDataLOGIN($datenFromHTTP{"login"}, $datenFromConfig{"userFile"}, $datenFromHTTP{"projekt"}); # use webFrame::userHandling; 

            ## User existiert
            if($userDataString ne "no")
            {
                  ## String 2 Hash
                  %datenFromUserDB = %{$userDataString};
                  
                  ## ist die sessionID richtig?
                  if($datenFromHTTP{"sessionID"} eq $datenFromUserDB{"sessionID"}){$korrektAngemeldet = "yes"; setTimeStamp($datenFromUserDB{"ID"}, $datenFromConfig{"userFile"}, $datenFromHTTP{"projekt"}); } # use webFrame::userHandling;
                  
                  ## und wie schauts mit der SessionID??
                  $sessionID = $datenFromUserDB{"sessionID"};
            }
      }
      ## Anmeldung erfolgt ber UserName und Passwd
      else
      {
            ## Das Passwd wird hier verschlsselt (wenn gewnscht)
            if($datenFromConfig{"cryptPasswd"} eq "1"){$datenFromHTTP{"passwd"} = ownCrypt($datenFromHTTP{"passwd"});}
            
            ## gibt es den USER in der DB, hier steht entweder ein
            ## "no" dann gibt es ihn nicht, oder ein Hash als String
            ## mit allen Werten
            my $userDataString = getUserDataLOGIN($datenFromHTTP{"login"}, $datenFromConfig{"userFile"}, $datenFromHTTP{"projekt"}); # use webFrame::userHandling; 
            
            ## User existiert
            if($userDataString ne "no")
            {
                  ## String 2 Hash
                  %datenFromUserDB = %{$userDataString};
            
                  ## ist das Passwort richtig?
                  if($datenFromHTTP{"passwd"} eq $datenFromUserDB{"passwd"}){$korrektAngemeldet = "yes"; setTimeStamp($datenFromUserDB{"ID"}, $datenFromConfig{"userFile"}, $datenFromHTTP{"projekt"}); } # use webFrame::userHandling;
                  
                  ## setzen der ID
                  $sessionID = setSessionID($datenFromUserDB{"ID"}, $datenFromConfig{"userFile"}, $datenFromHTTP{"projekt"}, $datenFromConfig{"lengthsessionID"}); # use webFrame::userHandling; 
            }
      }

      ## Wichtige Informationen werden in das Main Hash gedreht.
      $datenFromSUBMain{"korrektAngemeldet"} = $korrektAngemeldet;
      $datenFromSUBMain{"sessionID"} = $sessionID;
      
      ## Protokollierung, falls erwnscht
      if($datenFromConfig{"protOn"} eq "1"){writeSessionLog($datenFromHTTP{"projekt"}, $datenFromConfig{"protFile"}, $datenFromHTTP{"login"}, $datenFromENV{"REMOTE_ADDR"}, generateFormatTimeStamp(), $korrektAngemeldet, $datenFromHTTP{"action"}, $datenFromHTTP{"sourcePage"});} # use webFrame::userHandling
      
      ####################################
      ##
      ## Nachfolgend knnen die einzelnen, fr die Applikation bentigten Schritte
      ## eingepflegt werden. Einige Beispielssschritte sind bereits enthalten.
      ##
      ##  ||
      ##  \/
      ##
      ####################################
      
      ## Nicht erfolgreich angemeldet
      if($korrektAngemeldet eq "no")
      {
            ## welche Seite soll angezeigt werden
            $datenFromSUBMain{"useHTMLResponsePage"} = "normalErrorPage";

            ## und wie ist der Fehlertext?
            $datenFromSUBMain{"errorMessage"} = "<p>Dem System gelang es nicht, sie anzumelden. Dies kann mehrere Ursachen haben:</p><ul><li>Es wird strikt zwischen Gro/Kleinschreibung unterschieden. Haben Sie vielleicht nicht darauf geachtet? Ist ihre Umschalttast gedrckt?</li><li>Ihre sessionID ist ungltig / abgelaufen / unbekannt. Melden Sie sich bitte erneut an das System an.</li></ul>";
      }
      else
      {
            ################
            ##
            ## nachfolgend werden
            ## die Programmschritte aufgefhrt, wer bis
            ## hierher kommt, ist bereits richtig angemeldet
            ##
            ## es fehlt noch eine bessere Unterscheidung nach Projekt 
            ##
            ################
            
            ################
            ## AdminMain Seite aufbauen, wenn ein Admin sich gerade angemeldet hat und dieses lt. CFG Datei zulssig ist
            if($datenFromUserDB{"rights"} eq "1" && $datenFromHTTP{"sourcePage"} eq "anmeldung")
            {
                  ## Darf der Admin lt. CFG - Datei etwas machen?
                  if($datenFromConfig{"adminExist"} eq "1")
                  {
                        $datenFromSUBMain{"useHTMLResponsePage"} = "AdminMainPage";
                        
                        ## Standartinformationen fr jede Seite
                        %datenForTheHTMLEnv =   ("action" => "selectEditPage", "sourcePage" => "AdminMainPage","login" => $datenFromHTTP{"login"},"sessionID" => $sessionID,"projekt" => $datenFromHTTP{"projekt"});
                        
                  }
                  ## oder vielleicht doch eher nicht, dann folgt Fehlerseite + Text
                  else{$datenFromSUBMain{"useHTMLResponsePage"} = "normalErrorPage"; $datenFromSUBMain{"errorMessage"} = "In dem File ".$datenFromHTTP{"projekt"}.".cfg ist eingetragen, das der Admin ber keine Administrationsrechte verfgt. Entsprechend ist der Zugriff verweigert."; }
                  
            }
            
            ################
            ## Admin editSomething Seite aufbauen, wenn ein Admin das wnscht hat und dieses lt. CFG Datei zulssig ist
            if($datenFromUserDB{"rights"} eq "1" && $datenFromHTTP{"sourcePage"} eq "AdminMainPage")
            {

                  ## Darf der Admin lt. CFG - Datei etwas machen?
                  if($datenFromConfig{"adminExist"} eq "1")
                  {
                        
                        #############################
                        ## was soll Editiert werden?
                        
                        ## Config
                        if($datenFromHTTP{"editSomething"} eq "config")
                        {
                              $datenFromSUBMain{"useHTMLResponsePage"} = "AdminEditConfigPage";
                              
                              ## Standartinformationen fr jede Seite
                              %datenForTheHTMLEnv =   ("action" => "saveConfig", "sourcePage" => "AdminEditConfigPage","login" => $datenFromHTTP{"login"},"sessionID" => $sessionID,"projekt" => $datenFromHTTP{"projekt"});
                        }
                        ## Other
                        else
                        {
                              $datenFromSUBMain{"useHTMLResponsePage"} = "AdminEditDBPage";
                              $datenFromSUBMain{"nameOfDB2Edit"} = $datenFromHTTP{"editSomething"};
                              
                              ## Inhalt auslesen
                              $datenFromSUBMain{"oldContent"} = readDataFromDBFile("./webFrame/".$datenFromHTTP{"projekt"}."/",$datenFromHTTP{"editSomething"}); # use webFrame::fileOperation.pm; 
                              
                              ## Standartinformationen fr jede Seite
                              %datenForTheHTMLEnv =   ("nameOfDB2Edit" => $datenFromHTTP{"editSomething"}, "action" => "saveDB", "sourcePage" => "AdminEditDBPage","login" => $datenFromHTTP{"login"},"sessionID" => $sessionID,"projekt" => $datenFromHTTP{"projekt"});
                        }
                        
                        ## falls nichts angegeben war, Fehlerseite
                        if($datenFromHTTP{"editSomething"} eq ""){$datenFromSUBMain{"useHTMLResponsePage"} = "normalErrorPage"; $datenFromSUBMain{"errorMessage"} = "Sie mssen angeben, was Sie editieren wollen."; }
                        
                  }
                  ## oder vielleicht doch eher nicht, dann folgt Fehlerseite + Text
                  else{$datenFromSUBMain{"useHTMLResponsePage"} = "normalErrorPage"; $datenFromSUBMain{"errorMessage"} = "In dem File ".$datenFromHTTP{"projekt"}.".cfg ist eingetragen, das der Admin ber keine Administrationsrechte verfgt. Entsprechend ist der Zugriff verweigert."; }
                  
            }
            
            ################
            ## Admin will ConfigFile speichern
            if($datenFromUserDB{"rights"} eq "1" && $datenFromHTTP{"sourcePage"} eq "AdminEditConfigPage")
            {
                  ## Darf der Admin lt. CFG - Datei etwas machen?
                  if($datenFromConfig{"adminExist"} eq "1")
                  {
                        ## nach Speichern zurck zur AdminMainPage
                        $datenFromSUBMain{"useHTMLResponsePage"} = "AdminMainPage";
                        
                        ## Standartinformationen fr jede Seite
                        %datenForTheHTMLEnv =   ("sourcePage" => "AdminMainPage","login" => $datenFromHTTP{"login"},"sessionID" => $sessionID,"projekt" => $datenFromHTTP{"projekt"});
                                                      
                        ## der Clou, d.h. abspeichern:
                        writeData2FileHTTP($datenFromHTTPString); # use webFrame::fileOperation;
                  }
                  ## oder vielleicht doch eher nicht, dann folgt Fehlerseite + Text
                  else{$datenFromSUBMain{"useHTMLResponsePage"} = "normalErrorPage"; $datenFromSUBMain{"errorMessage"} = "In dem File ".$datenFromHTTP{"projekt"}.".cfg ist eingetragen, das der Admin ber keine Administrationsrechte verfgt. Entsprechend ist der Zugriff verweigert."; }

            }
            
            ################
            ## Admin will DBFile speichern
            if($datenFromUserDB{"rights"} eq "1" && $datenFromHTTP{"sourcePage"} eq "AdminEditDBPage")
            {
                  ## Darf der Admin lt. CFG - Datei etwas machen?
                  if($datenFromConfig{"adminExist"} eq "1")
                  {
                        ## nach Speichern zurck zur AdminMainPage
                        $datenFromSUBMain{"useHTMLResponsePage"} = "AdminMainPage";
                        
                        ## Standartinformationen fr jede Seite
                        %datenForTheHTMLEnv =   ("sourcePage" => "AdminMainPage","login" => $datenFromHTTP{"login"},"sessionID" => $sessionID,"projekt" => $datenFromHTTP{"projekt"});

                        ## der Clou, d.h. abspeichern:
                        writeData2DBFileHTTP($datenFromHTTPString, $datenFromConfig{"cryptPasswd"}); # use webFrame::fileOperation;
                  }
                  ## oder vielleicht doch eher nicht, dann folgt Fehlerseite + Text
                  else{$datenFromSUBMain{"useHTMLResponsePage"} = "normalErrorPage"; $datenFromSUBMain{"errorMessage"} = "In dem File ".$datenFromHTTP{"projekt"}.".cfg ist eingetragen, das der Admin ber keine Administrationsrechte verfgt. Entsprechend ist der Zugriff verweigert."; }

            }
      
            ################
            ## AnalyseSeite, falls bis jetzt noch nichts passiert ist.
            ## sollte spter noch gendert werden auf einen bestimmten Fall, nicht auf
            ## wenn nix passt.
            if($datenFromSUBMain{"useHTMLResponsePage"} eq ""){$datenFromSUBMain{"useHTMLResponsePage"} = "Analyse";}
      }
            
      ####################################
      ##
      ##  /\ 
      ##  ||
      ##
      ## Der Rest sollte wieder unangetastet bleiben.
      ##
      ####################################

      ## einpflegen wichtiger Daten (in Main erstellt)
      $hashDaten2Print{"datenFromSUBMain"} = \%datenFromSUBMain;

      ## einpflegen der anderen Hashes
      $hashDaten2Print{"datenFromConfig"} = \%datenFromConfig;
      $hashDaten2Print{"datenFromHTTP"} = \%datenFromHTTP;
      $hashDaten2Print{"datenFromENV"} = \%datenFromENV;
      $hashDaten2Print{"datenFromUserDB"} = \%datenFromUserDB;
      $hashDaten2Print{"datenForTheHTMLEnv"} = \%datenForTheHTMLEnv;
            
      return(\%hashDaten2Print);
}